import logging
import os

from pip._internal.utils.subprocess import runner_with_spinner_message
from pip._internal.utils.typing import MYPY_CHECK_RUNNING

if MYPY_CHECK_RUNNING:
    from typing import List, Optional
    from pip._vendor.pep517.wrappers import Pep517HookCaller

logger = logging.getLogger(__name__)


def build_wheel_pep517(
    name,  # type: str
    backend,  # type: Pep517HookCaller
    metadata_directory,  # type: str
    build_options,  # type: List[str]
    tempd,  # type: str
):
    # type: (...) -> Optional[str]
    """Build one InstallRequirement using the PEP 517 build process.

    Returns path to wheel if successfully built. Otherwise, returns None.
    """
    assert metadata_directory is not None
    if build_options:
        # PEP 517 does not support --build-options
        logger.error('Cannot build wheel for %s using PEP 517 when '
                     '--build-option is present' % (name,))
        return None
    try:
        logger.debug('Destination directory: %s', tempd)

        runner = runner_with_spinner_message(
            'Building wheel for {} (PEP 517)'.format(name)
        )
        with backend.subprocess_runner(runner):
            wheel_name = backend.build_wheel(
                tempd,
                metadata_directory=metadata_directory,
            )
    except Exception:
        logger.error('Failed building wheel for %s', name)
        return None
    return os.path.join(tempd, wheel_name)



import datetime
from .utils import format_rfc3339

try:
    _string_types = (str, unicode)
    _int_types = (int, long)
except NameError:
    _string_types = str
    _int_types = int

def translate_to_test(v):
    if isinstance(v, dict):
        return { k: translate_to_test(v) for k, v in v.items() }
    if isinstance(v, list):
        a = [translate_to_test(x) for x in v]
        if v and isinstance(v[0], dict):
            return a
        else:
            return {'type': 'array', 'value': a}
    if isinstance(v, datetime.datetime):
        return {'type': 'datetime', 'value': format_rfc3339(v)}
    if isinstance(v, bool):
        return {'type': 'bool', 'value': 'true' if v else 'false'}
    if isinstance(v, _int_types):
        return {'type': 'integer', 'value': str(v)}
    if isinstance(v, float):
        return {'type': 'float', 'value': '{:.17}'.format(v)}
    if isinstance(v, _string_types):
        return {'type': 'string', 'value': v}
    raise RuntimeError('unexpected value: {!r}'.format(v))
